## 
## Template for converting per-text counts to features (relative frequencies)
## (C) 2021 Stefan Evert
##

## read all tables tbl/*.tsv with per-text frequency counts
filenames <- sort(Sys.glob("tbl/*.tsv"))
if (!length(filenames)) stop("No files matching tbl/*.tsv found. Aborted.")
ICE.list <- lapply(filenames, function (fn) read.delim(fn, check.names=FALSE, stringsAsFactors=FALSE))
ICE <- do.call(rbind, ICE.list)

## check correct format
stopifnot(!any(duplicated(ICE$id)))         # IDs must be unique
stopifnot(all(sapply(ICE[-1], is.numeric))) # all columns must be numeric

## compute features
Features <- with(ICE,
  data.frame(
    id = id,
    sent = n_sent,
    token = n_token,
    word = n_word,
    word_S = n_word / n_sent,
    lexical_density = ld / n_word,
    nn_W = nn / n_word,
    np_W = np / n_word,
    nominal_W = nom / n_word,
    neoclass_W = neo / n_word,
    poss_pronoun_W = poss / n_word,
    pronoun_all_W = pronoun / n_word,
    p1_perspron_P = p1 / pronoun,
    p2_perspron_P = p2 / pronoun,
    p3_perspron_P = p3 / pronoun,
    it_P = pit / pronoun,
    pospers1_W = pospers1 / n_word,
    pospers2_W = pospers2 / n_word,
    pospers3_W = pospers3 / n_word,
    adj_W = adj / n_word,               # 3 collinear features,
    atadj_W = atadj / n_word,           # must select two of them 
    predadj_W = (adj - atadj) / n_word, # in data preparation
    prep_W = prep / n_word,
    finite_S = fin / n_sent,
    past_tense_F = past / fin,
    will_F = will / fin,
    modal_verb_V = vm / v,
    verb_W = v / n_word,
    infinitive_F = inf / fin,
    passive_F = pass / fin, 
    coordination_F = coord / fin, 
    subordination_F = subord / fin, 
    interrogative_S = interr / n_sent,
    imperative_S = imper / n_sent,
    title_W = title / n_word,
    salutation_S = salutgreet / n_sent,
    place_adv_W = rl / n_word,
    time_adv_W = rt / n_word,
    nom_initial_S = nptheme / n_sent,
    prep_initial_S = pptheme / n_sent,
    adv_initial_S = advtheme / n_sent,
    text_initial_S = cctheme / n_sent,
    wh_initial_S = whtheme / n_sent,
    disc_initial_S = disctheme / n_sent,
    nonfin_initial_S = totheme /n_sent,
    subord_initial_S = subordtheme / n_sent,
    verb_initial_S = verbtheme / n_sent,
    stringsAsFactors=FALSE
  )
)

## check for value errors
idx <- apply(as.matrix(Features[, -1]), 1, function (x) any(is.na(x)))
if (any(idx)) {
  cat(sprintf("Warning: %d rows have invalid features\n", sum(idx)))
  print(Features[idx, , drop=FALSE], digits=3)
  cat("Setting invalid feature scores to zero ...\n")
  na.zero <- function (x) { if (is.numeric(x)) x[is.na(x)] <- 0; x }
  Features <- as.data.frame(lapply(Features, na.zero), stringsAsFactors=FALSE)
}

## save complete feature matrix in .tsv format
write.table(Features, file="ice_features.tsv", quote=FALSE, sep="\t", row.names=FALSE, col.names=TRUE)
